#

  
/* UCLA Data Secure Unix

   The following C language preprocessor statements define a modified
   C language which is more readable than the standard C language.

   Programs written in this modified language should have the statement
   #include <Modified_cee.h>
   at the beginning.  When programs written in the modified language are
   shipped to another installation, this preprocessor file should also be
   shipped.
*/

/*
   Copyright 1978 by Mark Kampe, Charles Kline, Gerald Popek,
   Allen Stoughton, Michael Urban, and Evelyn Walton.

   Modifications by Steve Kelem Nov. 1981.
   Added type definition for STRING.
   Fixed value of Boolean TRUE so that it will work on any sized word.
   Corrected the capitalization of boolean to Boolean.

   1/20/81 (SHK) - Added UNION
   1/21/81 (SHK) - appended a ";" to break and continue
   2/8/82  (SHK) - fixed values for Booleans so that the correct size will be used when passed as a parameter
   2/10/82 (SHK) - added SHORT data type.
   89/04/10 (shk) - added XOR definition
   89/08/24 (shk) - added CONST, VOLATILE, and SIGNED
*/


/* Standard types */

#define AUTO		auto		/* standard automatic */
#define BOOL		char		/* Boolean values (TRUE/FALSE) */
#define CHAR		char		/* standard character */
#define CONST		const		/* constant value */
#define DOUBLE		double		/* standard double */
#define ENUM		enum		/* enumerated data type */
#define EXTERN		extern		/* standard external */
#define FLOAT		float		/* standard float */
#define INT		int		/* standard integer */
#define INTERN		intern		/* standard internal */
#define LONG		long		/* standard long */
#define PROC		/**/		/* used in procedure declarations */
#define REF		*		/* pointer declarations */
#define REGISTER	register	/* standard register */
#define SHORT		short		/* standard short */
#define SIGNED		signed		/* signed variables */
#define STATIC		static		/* standard static */
#define STRUCT		struct		/* standard structures */
#define TYPEDEF		typedef		/* standard typedef */
#define	UNION		union		/* standard union */
#define UNSIGNED	unsigned	/* unsigned variables */
#define VOID		/**/		/* used for procedure declarations
					   in which nothing is returned.  */
#define	VOLATILE	volatile	/* volatile storage */

/* New type */

typedef  CHAR REF STRING;		/* used for "char *" */


/* flow of control constructs */

#define GOTO		goto		/* standard goto */

#define BEGIN		{		/* begin - end pair: */
#define END		;}		/* BEGIN <stmts> END */

#define RETURN		return		/* standard return: */
					/* RETURN(<expression>) */
#define BREAK		;break;		/* standard break */
#define CONTINUE	continue;	/* standard continue */

#define IF		if(		/* if statement: */
					/* IF <bool expression>
					   THEN <statements>
					   ELSE <statements>
					   FI

					   (or)

					   IF <b-exp> THEN <stmts>
					   ELIF <b-exp> THEN <stmts>
						....
					   ELIF <b-exp> THEN <stmts>
					   ELSE <stmts>
					   FI

					   (where the ELSE part is optional)
					*/
#define THEN		){
#define ELSE		;}else{
#define ELIF		;}else if(
#define FI		;}

#define FOR		for(		/* for statement: */
					/* FOR <stmt> AS <bool expression>
					       EXEC <stmt>
					   DO  <statements>  OD

					   (where AS is short for ASLONGAS)
					*/
#define ASLONGAS	;(
#define AS		;(
#define EXEC		);
#define DO		){
#define OD		;}

#define WHILE		while(		/* while statement: */
					/* WHILE <bool expression>
					       DO  <statements> OD
					*/

#define	UNTIL		do {		/* UNTIL <statements> */
					/* UN <Bool expression> TIL */
#define	UN		;} while (
#define	TIL		);
#define SWITCH		switch(		/* switch statement: */
					/* SWITCH <expression>
					       IN
						   CASE <constant>: <stmts>
						       BREAK;
						   ....
						   DEFAULT: <stmts>
					       NI
					*/
#define CASE		case
#define IN		){
#define NI		;}
#define DEFAULT		default


/* Values */

#define TRUE		(char)~0	/* Boolean true */
#define FALSE		(char)0		/* Boolean false */
#define NORMAL		0		/* integer "normal" */
#define ERROR		-1		/* integer "error" */
#define UNDEFINED	-1		/* any type "undefined" */

#ifndef NULL
#define NULL		0		/* 0 of any type */
#endif                                  /* '\0' of character type */


/* Standard operators */
/* (it is suggested that the following english equivalents to the standard
    C operators be used in order to increase program readabilty.  However,
    in some cases the standard operators should probably still be used.)
*/

#define SIZEOF		sizeof		/* standard sizeof operator */
#define CONT		*		/* value of a pointer */
#define	NOT		!		/* standard logical negation */
#define	INCR		++		/* standard increment operator */
#define	DECR		--		/* standard decrement operator */
#define ADDR		&		/* address of an object */
#define MOD		%		/* standard mod operator */
#define ANDIF		&&		/* logical and -- */
					/* evaluate second operand only if
					   first is TRUE.
					*/
#define ORIF		||		/* logical or -- */
					/* evaluate second operand only if
					   first is FALSE.
					*/
#define AND		&		/* bit and -- */
					/* if used with Boolean values,
					   evaluate both operands.
					*/
#define OR		|		/* bit or -- */
					/* if used with Boolean values,
					   evaluate both operands.
					*/
#define EXCLOR		^		/* bit exclusive or */
#define XOR		^		/* bit exclusive or */
#define COMPLEMENT	~		/* standard bit complement */
#define LSHIFT		<<		/* standard arithmetic shift left */
#define RSHIFT		>>		/* standard arithmetic shift right */

/* Run time assertions */
/* The construct:
	ASSERT(<Boolean-expression>)
   will generate a run time check that the <Boolean-expression> is TRUE.
   If the expression is FALSE, the routine "assert_error" will be called.
   The standard C library version of this routine prints a fatal error
   message and causes an core image to be produced.
   When non UNIX user code is being written, a new version of assert_error
   should be used.

   To turn-off the generation of these run-time checks the compiler should
   be invoked as follows:
	cc <normal-options> -DNO_ASSERTIONS <program source>.
   This causes the compiler to make the symbol NO_ASSERTIONS defined.  If
   NO_ASSERTIONS is defined then run-time code will NOT be generated for
   the assertions -- making them effectively comments.
*/

#ifndef NO_ASSERTIONS
#define ASSERT(e)   IF NOT(e) \
THEN fprintf(stderr,"Assertion failed: file %s, line %d\n", __FILE__, __LINE__);\
exit(1)\
FI
#endif
#ifdef NO_ASSERTIONS
#define ASSERT(e)	/* */
#endif
